
varying vec3 position;
varying vec3 normal;
varying vec3 camera_dir;


float falloff(vec3 light_dir,
              vec3 spot_dir,
              float cos_cutoff,
              float spot_exponent)
{
	const float cos_outer_cone_angle = 0.8; // 36 degrees 

	vec3 spot_direction = normalize(spot_dir);
	float cos_cur_angle = dot(spot_dir,-light_dir);
	float cos_inner_cone_angle = cos_cutoff;
	float cos_inner_minus_outer_angle =  cos_inner_cone_angle - cos_outer_cone_angle;
	float falloff = clamp((cos_cur_angle - cos_outer_cone_angle) / cos_inner_minus_outer_angle,0.0,1.0);
	float spot = pow(cos_cur_angle,spot_exponent);
	return falloff * spot;
}

vec4 shade_spot_light_fade(int light_index,
						         vec3 light_pos,
						         vec3 spot_dir,
						         vec3 camera_dir,
						         vec3 vertex,
						         vec3 normal)
{
	// Resulting color
   vec4 result = vec4(0.0);
    
	// Compute direction to the light & the distance
   vec3  light_dir = light_pos - vertex;
   float distance = length(light_dir);
   light_dir = normalize(light_dir);

   // Compute attenuation
   float att = 1.0 / (gl_LightSource[light_index].quadraticAttenuation * distance * distance + 
                      gl_LightSource[light_index].linearAttenuation    * distance + 
                      gl_LightSource[light_index].constantAttenuation);
                                  
	// Spotlight part
	if (gl_LightSource[light_index].position.w < 1.0) {
		att = att * falloff(
			light_dir,
			spot_dir,
			gl_LightSource[light_index].spotCosCutoff,
			gl_LightSource[light_index].spotExponent
		);
	}
	
	// Compute half vector
	vec3 half_vec = normalize(light_dir + camera_dir);
	
	// Compute diffuse & specular part
	float diffuse  = max(dot(light_dir,normal),0.0);
	float specular = pow(max(dot(normal,half_vec),0.0),gl_FrontMaterial.shininess);
	
   // Combine color
	result += gl_FrontMaterial.ambient * gl_LightSource[light_index].ambient;
	result += att * gl_FrontMaterial.diffuse * gl_LightSource[light_index].diffuse * diffuse;
	result += att * gl_FrontMaterial.specular * gl_LightSource[light_index].specular * specular;
	
	return result;
}

vec4 shade_spot_light_fade_bump(int light_index,
                                mat3 tangent_mat,
                                vec3 light_pos,
                                vec3 spot_dir,
                                vec3 camera_dir,
                                vec3 vertex,
                                vec3 normal)
{
	// Resulting color
   vec4 result = vec4(0.0);
    
	// Compute direction to the light & the distance
   vec3  light_dir = light_pos - vertex;
   float distance = length(light_dir);
   light_dir = normalize(light_dir);

   // Compute attenuation
   float att = 1.0 / (gl_LightSource[light_index].quadraticAttenuation * distance * distance + 
                      gl_LightSource[light_index].linearAttenuation    * distance + 
                      gl_LightSource[light_index].constantAttenuation);
                                  
	// Spotlight part
	if (gl_LightSource[light_index].position.w < 1.0) {
		att = att * falloff(
			light_dir,
			spot_dir,
			gl_LightSource[light_index].spotCosCutoff,
			gl_LightSource[light_index].spotExponent
		);
	}
	
	// Transform light direction to tangent space and compute half vector
	light_dir = normalize(light_dir * tangent_mat);
	vec3 half_vec = normalize(light_dir + camera_dir);
	
	// Compute diffuse & specular part
	float diffuse  = max(dot(light_dir,normal),0.0);
	float specular = pow(max(dot(normal,half_vec),0.0),gl_FrontMaterial.shininess);
	
	// Combine color
	result += gl_FrontMaterial.ambient * gl_LightSource[light_index].ambient;
	result += att * gl_FrontMaterial.diffuse * gl_LightSource[light_index].diffuse * diffuse;
	result += att * gl_FrontMaterial.specular * gl_LightSource[light_index].specular * specular;
	
	return result;
}




void main()
{
	vec3 light_pos = (gl_ModelViewMatrixInverse * gl_LightSource[0].position).xyz;
	vec3 spot_dir = (gl_ModelViewMatrixInverse * vec4(gl_LightSource[0].spotDirection,0)).xyz;
	
	gl_FragColor = shade_spot_light_fade(
		0,
		light_pos,
		normalize(spot_dir),
		normalize(camera_dir),
		position,
		normalize(normal)
	);
}
